@@ -108,12 +108,6 @@ sub get {
   # Nonfatal sysread() error.  Return an empty list.
   return [ ] if $! == EAGAIN or $! == EWOULDBLOCK;
 
-  # In perl 5.005_04 on FreeBSD, $! is not set properly unless this
-  # silly no-op is executed.  Turn off warnings in case $result isn't
-  # defined.  TODO - Make it optimizable at compile time.
-  local $^W = 0;
-  $result = "$result";
-
   # fatal sysread error
   undef;
 }
