@@ -15,22 +15,28 @@ use strict;
 ### The count of all extra references used in the system.
 
 my %kr_extra_refs;
-#  ( $session =>
+#  ( $session_id =>
 #    { $tag => $count,
 #       ...,
 #     },
 #     ...,
 #   );
 
+sub _data_extref_relocate_kernel_id {
+  my ($self, $old_id, $new_id) = @_;
+  return unless exists $kr_extra_refs{$old_id};
+  $kr_extra_refs{$new_id} = delete $kr_extra_refs{$old_id};
+}
+
 ### End-run leak checking.
 
 sub _data_extref_finalize {
   my $finalized_ok = 1;
-  foreach my $session (keys %kr_extra_refs) {
+  foreach my $session_id (keys %kr_extra_refs) {
     $finalized_ok = 0;
-    _warn "!!! Leaked extref: $session\n";
-    foreach my $tag (keys %{$kr_extra_refs{$session}}) {
-      _warn "!!!\t`$tag' = $kr_extra_refs{$session}->{$tag}\n";
+    _warn "!!! Leaked extref: $session_id\n";
+    foreach my $tag (keys %{$kr_extra_refs{$session_id}}) {
+      _warn "!!!\t`$tag' = $kr_extra_refs{$session_id}->{$tag}\n";
     }
   }
   return $finalized_ok;
@@ -51,19 +57,19 @@ sub _data_extref_finalize {
 # refcount loops and possibly prevent that.
 
 sub _data_extref_inc {
-  my ($self, $session, $tag) = @_;
-  my $refcount = ++$kr_extra_refs{$session}->{$tag};
+  my ($self, $sid, $tag) = @_;
+  my $refcount = ++$kr_extra_refs{$sid}->{$tag};
 
   # TODO We could probably get away with only incrementing the
   # session's master refcount once, as long as any extra refcount is
   # positive.  Then the session reference count would be a flag
   # instead of a counter.
-  $self->_data_ses_refcount_inc($session) if $refcount == 1;
+  $self->_data_ses_refcount_inc($sid) if $refcount == 1;
 
   if (TRACE_REFCNT) {
     _warn(
       "<rc> incremented extref ``$tag'' (now $refcount) for ",
-      $self->_data_alias_loggable($session)
+      $self->_data_alias_loggable($sid)
     );
   }
 
@@ -78,74 +84,73 @@ sub _data_extref_inc {
 # Hopefully the public interface won't allow it.
 
 sub _data_extref_dec {
-  my ($self, $session, $tag) = @_;
+  my ($self, $sid, $tag) = @_;
 
   if (ASSERT_DATA) {
     # Prevents autoviv.
     _trap("<dt> decrementing extref for session without any")
-      unless exists $kr_extra_refs{$session};
+      unless exists $kr_extra_refs{$sid};
 
-    unless (exists $kr_extra_refs{$session}->{$tag}) {
+    unless (exists $kr_extra_refs{$sid}->{$tag}) {
       _trap(
         "<dt> decrementing extref for nonexistent tag ``$tag'' in ",
-        $self->_data_alias_loggable($session)
+        $self->_data_alias_loggable($sid)
       );
     }
   }
 
-  my $refcount = --$kr_extra_refs{$session}->{$tag};
+  my $refcount = --$kr_extra_refs{$sid}->{$tag};
 
   if (TRACE_REFCNT) {
     _warn(
       "<rc> decremented extref ``$tag'' (now $refcount) for ",
-      $self->_data_alias_loggable($session)
+      $self->_data_alias_loggable($sid)
     );
   }
 
-  $self->_data_extref_remove($session, $tag) unless $refcount;
+  $self->_data_extref_remove($sid, $tag) unless $refcount;
   return $refcount;
 }
 
 ### Remove an extra reference from a session, regardless of its count.
 
 sub _data_extref_remove {
-  my ($self, $session, $tag) = @_;
+  my ($self, $sid, $tag) = @_;
 
   if (ASSERT_DATA) {
     # Prevents autoviv.
     _trap("<dt> removing extref from session without any")
-      unless exists $kr_extra_refs{$session};
-    unless (exists $kr_extra_refs{$session}->{$tag}) {
+      unless exists $kr_extra_refs{$sid};
+    unless (exists $kr_extra_refs{$sid}->{$tag}) {
       _trap(
         "<dt> removing extref for nonexistent tag ``$tag'' in ",
-        $self->_data_alias_loggable($session)
+        $self->_data_alias_loggable($sid)
       );
     }
   }
 
-  delete $kr_extra_refs{$session}->{$tag};
-  delete $kr_extra_refs{$session}
-    unless scalar keys %{$kr_extra_refs{$session}};
-  $self->_data_ses_refcount_dec($session);
+  delete $kr_extra_refs{$sid}->{$tag};
+  delete $kr_extra_refs{$sid} unless scalar keys %{$kr_extra_refs{$sid}};
+  $self->_data_ses_refcount_dec($sid);
 }
 
 ### Clear all the extra references from a session.
 
 sub _data_extref_clear_session {
-  my ($self, $session) = @_;
+  my ($self, $sid) = @_;
 
   # TODO - Should there be a _trap here if the session doesn't exist?
 
-  return unless exists $kr_extra_refs{$session}; # avoid autoviv
-  foreach (keys %{$kr_extra_refs{$session}}) {
-    $self->_data_extref_remove($session, $_);
+  return unless exists $kr_extra_refs{$sid}; # avoid autoviv
+  foreach (keys %{$kr_extra_refs{$sid}}) {
+    $self->_data_extref_remove($sid, $_);
   }
 
   if (ASSERT_DATA) {
-    if (exists $kr_extra_refs{$session}) {
+    if (exists $kr_extra_refs{$sid}) {
       _trap(
         "<dt> extref clear did not remove session ",
-        $self->_data_alias_loggable($session)
+        $self->_data_alias_loggable($sid)
       );
     }
   }
@@ -161,9 +166,9 @@ sub _data_extref_count {
 # Fetch whether a session has extra references.
 
 sub _data_extref_count_ses {
-  my ($self, $session) = @_;
-  return 0 unless exists $kr_extra_refs{$session};
-  return scalar keys %{$kr_extra_refs{$session}};
+  my ($self, $sid) = @_;
+  return 0 unless exists $kr_extra_refs{$sid};
+  return scalar keys %{$kr_extra_refs{$sid}};
 }
 
 1;
