@@ -4,7 +4,6 @@ use strict;
 
 use vars qw($VERSION @ISA);
 $VERSION = '1.299'; # NOTE - Should be #.### (three decimal places)
-@ISA = 'POE::Wheel';
 
 use Carp qw(carp croak);
 use POSIX qw(
@@ -13,7 +12,7 @@ use POSIX qw(
 );
 
 use POE qw( Wheel Pipe::TwoWay Pipe::OneWay Driver::SysRW Filter::Line );
-use base qw(POE::Wheel);
+push @ISA, qw(POE::Wheel);
 
 # http://rt.cpan.org/Ticket/Display.html?id=50068
 # Avoid using these constants in Windows' subprocesses (actually
@@ -26,10 +25,24 @@ BEGIN {
 
   local $SIG{'__DIE__'} = 'DEFAULT';
   eval    { require IO::Pty; };
-  if ($@) { eval 'sub PTY_AVAILABLE () { 0 }';  }
+  if ($@) {
+    eval '
+      sub PTY_AVAILABLE () { 0 }
+      sub TIOCSWINSZ_AVAILABLE () { 0 }
+    ';
+  }
   else {
     IO::Pty->import();
     eval 'sub PTY_AVAILABLE () { 1 }';
+
+    eval { require IO::Tty; };
+    if ($@) {
+      eval 'sub TIOCSWINSZ_AVAILABLE () { 0 }';
+    }
+    else {
+      IO::Tty->import('TIOCSWINSZ');
+      eval 'sub TIOCSWINSZ_AVAILABLE () { 1 }';
+    }
   }
 
   if (POE::Kernel::RUNNING_IN_HELL) {
@@ -159,9 +172,18 @@ sub new {
     $conduit = "pipe";
   }
 
-  # TODO remove deprecation warning after some time
-  carp "Winsize is deprecated." if exists $params{Winsize};
-  delete $params{Winsize};  # so the unknown arg check doesn't complain
+  my $winsize = delete $params{Winsize};
+
+  if ($winsize) {
+    carp "winsize can only be specified for a Conduit of type pty"
+      if $conduit ne 'pty' and $winsize;
+
+    carp "winsize must be a 4 element arrayref" unless ref($winsize) eq 'ARRAY'
+      and scalar @$winsize == 4;
+
+    carp "winsize only works when IO::Tty::TIOCSWINSZ is"
+      unless TIOCSWINSZ_AVAILABLE;
+  }
 
   my $stdin_event  = delete $params{StdinEvent};
   my $stdout_event = delete $params{StdoutEvent};
@@ -333,9 +355,17 @@ sub new {
       # per APITUE 19.4 and 11.10.
       $stdin_read->set_raw();
 
-      # Set the pty conduit (slave side) window size to our window
-      # size.  APITUE 19.4 and 19.5.
-      eval { $stdin_read->clone_winsize_from(\*STDIN) };
+      if (TIOCSWINSZ_AVAILABLE) {
+        if ($winsize) {
+          ioctl($stdin_read, TIOCSWINSZ, pack('vvvv', @$winsize));
+        }
+      }
+      else {
+        # Set the pty conduit (slave side) window size to our window
+        # size.  APITUE 19.4 and 19.5.
+
+        eval { $stdin_read->clone_winsize_from(\*STDIN) } if -T STDIN;
+      }
     }
     else {
       # TODO - Can this be block eval?  Or a do{} block?
@@ -1474,23 +1504,10 @@ TODO - Example.
 
 =head4 Winsize
 
-WARNING! This has been deprecated. WARNING!
-
-The reason for the deprecation is that the original code was crufty
-and unmaintained. It caused more problems than it helped. In the
-mists of time, the old code was silently removed. Actually, it was
-replaced with IO::Pty::clone_winsize_from(\*FH) which was more saner.
-Alas, the docs weren't updated and users were led to believe that
-this would do something. Now, POE::Wheel::Run will issue a warning
-if it sees this param. If you want the old behavior back, please help
-us fix it! Patches welcome :)
-
-WARNING! This has been deprecated. WARNING!
-
 Winsize sets the child process' terminal size.  Its value should be an
-arrayref with two or four elements.  The first two elements must be
-the number of lines and columns for the child's terminal window,
-respectively.  The optional pair of elements describe the terminal's X
+arrayref with four elements.  The first two elements must be the
+number of lines and columns for the child's terminal window,
+respectively.  The second pair of elements describe the terminal's X
 and Y dimensions in pixels:
 
   $_[HEAP]{child} = POE::Wheel::Run->new(
